/* 1 Dec 2002.

   Md5 loader class.
   Loads md5mesh and md5anim.
   tga textures need to be in the same directory.

   md5anim still needs improvement:
   - won't work with every anims.
   - animation may be wrong since I don't use
     ranges.. how to use them? interpolate keys in the range?
   - what is the 'string' property ??
   hum found this in scene1pinky.md5anim :
   
        strings 6
   "camera1" "camera1a" "camera2" "camera3" "camera4" "camera5"^M

   and following keys are between 0 and 5. will work
   on this later ;)


   this code is released under the GPL License

   (c) 2002 Sbastien Kuntz
   aka CbWan <cb@nowan.net>
*/

#ifndef __MD5_H__
#define __MD5_H__

#include <vector>   
#include <stdio.h>

struct Channel;

struct Bone
{
  char name[64];
  float pos[3];
  float rot[9];

  int parent;

  std::vector<Bone*> children;

  Channel* chan[6];
};

enum {
  X,Y,Z,YAW,PITCH,ROLL
};

struct Channel
{
  int nb;
  char joint[64];

  Bone* bone;

  int attribute;

  float starttime;
  float endtime;

  float framerate;
  int  strings;
  int  range[2];
  int  nbKeys;
  float* keys;
};

struct Vertex
{
   float u, v;
   int weight_index;
   int weight_count;
   float x, y, z;
   float nx, ny, nz;
};

struct Triangle
{
   int a, b, c;
};

struct Weight
{
   int bone;
   float t;
   float x, y, z;
};

struct Mesh
{
  char texture[512];
  int  textId;
  int          nb_vertices;
  Vertex*      vertices;
  int          nb_triangles;
  Triangle*    triangles;
  int          nb_weights;
  Weight*      weights;
};


class MD5 {
 public:
  MD5( const char* meshfilename, const char* animfilename );
  ~MD5( void );

  void setFrame( int frame );

  /* attributes. not private for simplicity, don't shout =) */

  unsigned int* _textures; /* textures id */
  int   _nbMeshes; 
  Mesh* _meshes;

  int   _nbBones;
  Bone* _bones;

  int   _nbChannels;
  Channel* _channels;

 private:
  Bone* findBone( const char* name );

  bool readLine( FILE* );

  bool readANIMFile(const char* filename);
  bool readDOOMFile(const char* filename);

  void recursiveBuildBones( int frame, Bone* bone );
  void buildNormals(void);
  void buildVertices(void);
  void buildTextures(void);
  
  bool _anim;

  char  _currentLine[1024];  
};

#endif
