/* md5viewer

   Usage : 
   
      md5viewer <md5mesh> [<md5anim>]

   Don't forget to put all the tga in the directory
   where you run md5viewer

   this code is released under the GPL License

   (c) 2002 Sbastien Kuntz
   aka CbWan <cb@nowan.net>
*/

#include "md5.h"

// Framework from nehe (nehe.gamedev.net)
#include <GL/glut.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include <unistd.h>

#include <iostream>
using namespace std;

#define ESCAPE 27

MD5* md5;

int window; 

float gx =    7.0f;
float gy =  -25.0f;
float gz = -170.0f;

float head  = -40;
float pitch =  20;

int animating = 0;
unsigned int frame = 0;

int startx = 0;
int starty = 0;

int moving  = 0;
int zooming = 0;
int panning = 0;

GLfloat lightAmbient []= {   0.0f,  0.0f, 0.0f, 1.0f };
GLfloat lightDiffuse []= {    .5f,   .5f,  .5f, 1.0f };
GLfloat lightPosition[]= { 100.0f,100.0f, 0.0f, 1.0f };
float   lightSpecular[]= {  -0.2f, -0.2f,-0.2f, 1.0f };

void
initGL( int width, int height )
{
  glClearColor(0.2f, 0.2f, 0.4f, 0.0f);
  glClearDepth(1.0);

  glDepthFunc(GL_LEQUAL);
  glEnable(GL_DEPTH_TEST);
  glShadeModel(GL_SMOOTH);
  glEnable(GL_CULL_FACE);
  
  glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);

  glLightfv(GL_LIGHT1, GL_POSITION, lightPosition);
  glLightfv(GL_LIGHT1, GL_AMBIENT,  lightAmbient);
  glLightfv(GL_LIGHT1, GL_DIFFUSE,  lightDiffuse);
  glLightfv(GL_LIGHT1, GL_SPECULAR, lightSpecular);
  glEnable(GL_LIGHT1);
  glEnable(GL_LIGHTING);

  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();

  gluPerspective(45.0f,(GLfloat)width/(GLfloat)height,0.1f,100.0f);

  glMatrixMode(GL_MODELVIEW);
  glColorMaterial(GL_FRONT, GL_AMBIENT_AND_DIFFUSE);
  glEnable(GL_COLOR_MATERIAL);
}

void
resizeGLScene(int width, int height)
{
  if (height==0)
    height=1;

  glViewport(0, 0, width, height);

  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();

  gluPerspective(45.0f,(GLfloat)width/(GLfloat)height,0.1f,1000.0f);
  glMatrixMode(GL_MODELVIEW);
}

void
drawAxis(void)
{
  glPushMatrix();
  glScalef(10,10,10);
  glDisable(GL_LIGHTING);
  glColor3f(1,0,0);
  glBegin( GL_LINES );
  glVertex3f(0,0,0);
  glVertex3f(1,0,0);
  glEnd();

  glColor3f(0,1,0);
  glBegin( GL_LINES );
  glVertex3f(0,0,0);
  glVertex3f(0,1,0);
  glEnd();

  glColor3f(0,0,1);
  glBegin( GL_LINES );
  glVertex3f(0,0,0);
  glVertex3f(0,0,1);
  glEnd();
  glEnable(GL_LIGHTING);
  glPopMatrix();
}

void
drawMd5Mesh( MD5* md5 )
{
  glColor3f(1,1,1);
  for( int n_mesh=0; n_mesh < md5->_nbMeshes; n_mesh++ ) {
    Mesh& mesh = md5->_meshes[ n_mesh ];
      
    if ( !md5->_textures[n_mesh] ) {
      glDisable(GL_TEXTURE_2D);
    }
    else {
      glEnable( GL_TEXTURE_2D );
      glBindTexture( GL_TEXTURE_2D, md5->_textures[n_mesh] );
    }

    glBegin(GL_TRIANGLES);

    for( int n_tri=0; n_tri < mesh.nb_triangles; n_tri++ ) {
      Triangle tri = mesh.triangles[ n_tri ];
		
      Vertex& v0 = mesh.vertices[tri.a];
      Vertex& v1 = mesh.vertices[tri.b];
      Vertex& v2 = mesh.vertices[tri.c];       
	
      glNormal3f  ( v0.nx, v0.ny, v0.nz );
      glTexCoord2f( v0.u , v0.v );
      glVertex3f  ( v0.x , v0.y , v0.z  );

      glNormal3f  ( v1.nx, v1.ny, v1.nz );
      glTexCoord2f( v1.u , v1.v );
      glVertex3f  ( v1.x , v1.y , v1.z  );

      glNormal3f  ( v2.nx, v2.ny, v2.nz );
      glTexCoord2f( v2.u, v2.v );
      glVertex3f  ( v2.x , v2.y , v2.z  );
	
    }

    glEnd();

  }
  glDisable(GL_TEXTURE_2D);
}

void
drawGLScene(void)
{
  glClear(GL_DEPTH_BUFFER_BIT | GL_COLOR_BUFFER_BIT | GL_STENCIL_BUFFER_BIT );

  glLoadIdentity();		

  // camera
  glTranslatef(gx,gy,gz);    
  glRotatef(pitch,1.0,0,0);
  glRotatef(head,0,1,0);

  drawAxis();

  md5->setFrame(frame);
  drawMd5Mesh(md5);
  
  glutSwapBuffers();
}

void keyPressed(unsigned char key, int x, int y) 
{
    usleep(100);

    switch( key ) 
    { 
    case ESCAPE:
      glutDestroyWindow(window);
      exit(0);  
      break;

    case 'f':
      frame--;
      cout << "frame : " << frame << endl;
      break;

    case 'g':
      frame++;
      cout << "frame : " << frame << endl;
      break;      

    case ' ':
      /* toggle between 1 and 0 */
      animating = 1 - animating;
      break;

    default:
      break;
    }    
}

void
mouse(int button, int state, int x, int y)
{
  /* panning */
  if(button == GLUT_RIGHT_BUTTON && state == GLUT_DOWN ) {
    panning = 1;
    startx = x;
    starty = y;
  }
  if(button == GLUT_RIGHT_BUTTON && state == GLUT_UP ) {
    panning = 0;
  }

  /* zooming */
  if(button == GLUT_MIDDLE_BUTTON && state == GLUT_DOWN) {
    zooming = 1;
    starty = y;
  }
  if (button == GLUT_MIDDLE_BUTTON && state == GLUT_UP) {
    zooming = 0;    
  }

  /* rotating */
  if (button == GLUT_LEFT_BUTTON && state == GLUT_DOWN) {
    moving = 1;
    startx = x;
    starty = y;
  }
  if (button == GLUT_LEFT_BUTTON && state == GLUT_UP) {
    moving = 0;
  }
}

void
motion(int x, int y)
{
  if (moving) {
    head = head + (x - startx);
    pitch = pitch + (y - starty);
    startx = x;
    starty = y;
    glutPostRedisplay();
  }
  if(zooming) {
    gz = gz + (y-starty);
    starty=y;
    glutPostRedisplay();
  }
  if(panning) {
    gx += x - startx;
    gy += - y + starty;
    glutPostRedisplay();
    startx=x;
    starty=y;
  }
}

void updateFrame( int value ) {
  if( animating ) {
    frame++;
  }

  /* rearm timer */
  glutTimerFunc(41, // 24 frames per second = 41ms per frame
                &updateFrame, 0);

}

int main(int argc, char **argv) 
{
  if( argc < 2 ) {
    cout << "Usage : " << endl << endl
	 << "  md5viewer <md5mesh> [<md5anim>]"
	 << endl << endl;
    exit(0);
  }

  if( argc > 2 )
    animating = 1;

  glutInit(&argc, argv);  
  glutInitDisplayMode(GLUT_RGBA | GLUT_DOUBLE | GLUT_ALPHA | GLUT_DEPTH);
  glutInitWindowSize(1024, 768);  
  glutInitWindowPosition(0, 0);

  window = glutCreateWindow("MD5 Viewer");  

  glutDisplayFunc(&drawGLScene);
  
  glutMouseFunc(mouse);
  glutMotionFunc(motion);

  glutIdleFunc(&drawGLScene);

  glutTimerFunc(41, // 24 frames per second = 41ms per frame
                &updateFrame, 0);


  glutReshapeFunc(&resizeGLScene);
  glutKeyboardFunc(&keyPressed);
  initGL(800, 600);

  md5 = new MD5( argv[1], argv[2] );

  glutMainLoop();  

  return 1;
}




